namespace Test {

	class TestItSuite : TestIt.Suite {

		private TestIt.Suite suite;

		public override void initialize () {
			name = "TestIt::Suite";
			add_test_case ("name", test_name);
			add_test_case ("success", test_success);
			add_test_case ("skip", test_skip);
			add_test_case ("failure", test_failure);
			add_test_case ("timeout", test_timeout);
			add_test_case ("assert_true success", test_assert_true_success);
			add_test_case ("assert_true failure", test_assert_true_failure);
			add_test_case ("assert_false success", test_assert_false_success);
			add_test_case ("assert_false failure", test_assert_false_failure);
			add_test_case ("async success", test_async_success);
			add_test_case ("async skip", test_async_skip);
			add_test_case ("async failure", test_async_failure);
			add_test_case ("async timeout", test_async_timeout);
		}

		public override void set_up () {
			suite = new TestIt.Suite();
			suite.name = "hello";
			suite.current_test_case = new TestIt.Case("hello");
		}

		public override void tear_down () {
			suite = null;
		}

		private void test_name () {
			assert_true (suite.name == "hello");
		}

		private void test_success () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.success ();
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 1);
		}

		private void test_skip () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.skip ();
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
		}

		private void test_failure () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.failure ();
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
		}

		private void test_timeout () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.timeout ();
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
		}

		private void test_assert_true_success () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.assert_true (1 == 1);
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 1);
		}

		private void test_assert_true_failure () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.assert_true (1 == 0);
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
		}

		private void test_assert_false_success () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.assert_false (1 == 0);
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 1);
		}

		private void test_assert_false_failure () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.assert_false (1 == 1);
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
		}

		private void test_async_success () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.async_success (); // TRICKY
			suite.async_wait (20);
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 1);
		}

		private void test_async_skip () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.async_skip (); // TRICKY
			suite.async_wait (20);
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
		}

		private void test_async_failure () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.async_failure (); // TRICKY
			suite.async_wait (20);
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
		}

		private void test_async_timeout () {
			assert_true (suite.current_test_case.total_assertions == 0);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
			suite.async_wait (20);
			assert_true (suite.current_test_case.total_assertions == 1);
			assert_true (suite.current_test_case.succeeded_assertions == 0);
		}


		static int main(string[] args) {
			var suite = new TestItSuite();
			return suite.run();
		}
	}
}
