using GLib;
using Gee;

using People.Utils;


namespace People.Utils.Test {

	private class AsyncOperationHelperExtension : People.Utils.AsyncOperationHelper {

		public Set<AsyncOperation> do_get_pending_operations() {
			return pending_operations;
		}

		public AsyncOperation<Collection> do_sync_result(Object object) {
			return sync_result(object);
		}

		public AsyncOperation<bool> do_sync_result_bool(bool value) {
			return sync_result_bool(value);
		}

		public AsyncOperation<void*> do_sync_error(Error error) {
			return sync_error(error);
		}


		public AsyncOperation<Object> do_async_operation_init() {
			return async_operation_init();
		}

		public void do_async_operation_emit(AsyncOperation<Object> op,
				Object object) {
			async_operation_emit(op, object);
		}


		public AsyncOperation<bool> do_async_operation_init_bool() {
			return async_operation_init_bool();
		}

		public void do_async_operation_emit_bool(AsyncOperation<bool> op, bool value) {
			async_operation_emit_bool(op, value);
		}

		public void do_async_operation_error(AsyncOperation<void*> op,
				Error error) {
			async_operation_error(op, error);
		}

	}

	private errordomain TestAsyncOperationHelperError {
		RANDOM_ERROR = 1,
	}

	class AsyncOperationHelper : TestIt.Suite {

		AsyncOperationHelperExtension<string> helper;

		protected override void initialize () {
			name = "People.Utils::AsyncOperationHelper";
			add_test_case ("synchronous object result", test_sync_object_result);
			add_test_case ("synchronous bool result", test_sync_bool_result);
			add_test_case ("synchronous error", test_sync_error);
			add_test_case ("asynchronous object result", test_async_object_result);
			add_test_case ("asynchronous bool result", test_async_bool_result);
			add_test_case ("asynchronous error", test_async_error);
		}

		protected override void set_up () {
			helper = new AsyncOperationHelperExtension ();
			base.set_up ();
		}

		protected override void tear_down () {
			helper = null;
			base.tear_down ();
		}

		private void test_sync_object_result () {
			Collection collection = new ArrayList<void*> ();

			assert_true (helper.do_sync_result (collection).get_result () == collection);
		}

		private void test_sync_bool_result () {
			assert_true (helper.do_sync_result_bool (true).get_result () == true);
			assert_true (helper.do_sync_result_bool (false).get_result () == false);
		}

		private void test_sync_error () {
			try {
				helper.do_sync_error(new TestAsyncOperationHelperError.RANDOM_ERROR("oops")).get_result();
				failure();
			} catch (TestAsyncOperationHelperError e) {
				success();
				assert_true (e.message == "oops");
			}
		}

		private void test_async_object_result () {
			assert_true (helper.do_get_pending_operations ().size == 0);

			var op = helper.do_async_operation_init ();
			assert_true (helper.do_get_pending_operations ().size == 1);

			Collection collection = new ArrayList<void*> ();
			helper.do_async_operation_emit (op, collection);
			assert_true ((Collection) op.get_result () == collection);
			assert_true (helper.do_get_pending_operations ().size == 0);
		}

		private void test_async_bool_result () {
			assert_true (helper.do_get_pending_operations ().size == 0);

			var op1 = helper.do_async_operation_init_bool ();
			assert_true (helper.do_get_pending_operations ().size == 1);

			helper.do_async_operation_emit_bool (op1, true);
			assert_true (op1.get_result () == true);
			assert_true (helper.do_get_pending_operations ().size == 0);

			var op2 = helper.do_async_operation_init_bool ();
			assert_true (helper.do_get_pending_operations ().size == 1);

			helper.do_async_operation_emit_bool (op2, false);
			assert_true (op2.get_result () == false);
			assert_true (helper.do_get_pending_operations ().size == 0);
		}

		private void test_async_error () {
			assert_true (helper.do_get_pending_operations().size == 0);

			var op = helper.do_async_operation_init_bool ();
			assert_true (helper.do_get_pending_operations ().size == 1);

			helper.do_async_operation_error(op, new TestAsyncOperationHelperError.RANDOM_ERROR("oops"));
			try {
				op.get_result ();
				failure ();
			} catch (TestAsyncOperationHelperError e) {
				success ();
				assert_true (e.message == "oops");
			}

			assert_true (helper.do_get_pending_operations ().size == 0);
		}

		static int main(string[] args) {
			var suite = new AsyncOperationHelper ();
			return suite.run();
		}

	}
}
